{This program loads a binary file from an SD card and then sends this file to
a second propeller}


CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  _TV_Enabled = true ' TV display on
  _TV_Pin = 16 ' group of 3 pins starting at pin 16

  _SD_DO = 12
  _SD_CLK = 13
  _SD_DI = 14
  _SD_CS = 15
  _SD_WP = -1 ' -1 ifnot installed.
  _SD_CD = -1 ' -1 ifnot installed.
  _RTC_DAT = -1 ' -1 ifnot installed.
  _RTC_CLK = -1 ' -1 ifnot installed.

  ' prop loader constants
  #1, ErrorConnect, ErrorVersion, ErrorChecksum, ErrorProgram, ErrorVerify
  #0, Shutdown, LoadRun, ProgramShutdown, ProgramRun  

OBJ
  text:  "tv_text"
  fat:   "SD-MMC_FatEngine_No_RTC.spin"  ' smaller version of Kye's fat engine with no RTC
  str:   "ASCII0_STREngine.spin"

VAR
   byte LineOfText1[80] ' general purpose string buffer
   ' prop loader constants
   long P31, P30, LFSR, Ver, Echo    

PUB Main 
  text.start(_TV_Pin)                                 ' start TV or VGA
  text.clearscreen
  PrintStringCR(string("Testing for SD card"))
  ifnot( fat.FATEngineStart(_SD_DO, _SD_CLK, _SD_DI, _SD_CS,_SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1))
    reboot
  PrintStringCR(string("SD found"))
  fat.mountPartition(0)     ' mount the sd card
  PrintStringCR(string("SD mounted"))    
  PrintStringCR(string("Open file"))
  fat.OpenFile(string("Bargraph.exe"),"R")     ' should be 8.3 format
  ConnectPropeller(0, 1, 2, 1, LoadRun, fat.filesize) ' Res,P31,P30,could read byte 8 and 9, OR use fat.filesize      
  fat.CloseFile
  PrintStringCR(string("Close file"))    
  'repeat

PUB ExtractCommandline    ' there is a file COMMAND.TXT that Kyedos program created to pass parameters
    ' need to add code eg to parse MYPROGRAM 1 2 3
    ' may wish to pass the file to send and the pins to send it on eg PROPLOAD MYFILE 0

PUB PrintStringCR(pstring)' sends pstring to the display
     text.str(pstring)  
     crlf

PUB PrintChar(c) ' sends c to the display
    text.out(c)
   
PUB crlf
    PrintChar(13)                                        ' send to display
    PrintChar(10)                                        ' send to display


' *************** prop loader code *******************

PUB ConnectPropeller(PinRESn, PinP31, PinP30, Version, Command, NumberBytes) : Error
  P31 := PinP31                                         'set P31 and P30 
  P30 := PinP30
  outa[PinRESn] := 0                                    ' RESn low
  dira[PinRESn] := 1                                    ' and set as output
  outa[PinP31] := 1                                     'P31 high (our TX)     
  dira[PinP31] := 1                                     ' and set as output
  dira[PinP30] := 0                                     'P30 input (our RX)   
  outa[PinRESn] := 1                                    'RESn high   
  waitcnt(clkfreq / 10 + cnt)                           ' wait 100ms
  Communicate(Version, Command, NumberBytes)            ' communicate
  dira[PinRESn] := 0                                    ' reset pin floating
  dira[PinP31] := 0                                     'P31 float
    

PRI Communicate(Version, Command, NumberBytes) | ByteCount
  BitsOut(%01, 2)            'output calibration pulses       
  LFSR := "P"                 'send LFSR pattern          
  repeat 250
    BitsOut(IterateLFSR, 1)
  repeat 250                     'receive and verify LFSR pattern    
    if WaitBit(1) <> IterateLFSR
      text.str(string("Error Connect"))
  repeat 8                  'receive chip version      
    Ver := WaitBit(1) << 7 + Ver >> 1
  text.str(string("Version "))
  text.dec(Ver)
  crlf
  if Ver <> Version                  'if version mismatch, shutdown and abort    
    BitsOut(Shutdown, 32)
    text.str(string("Error Version"))
  BitsOut(Command, 32)                                    'send command                              
  if Command                                              'handle command details     
    'ByteCount := byte[CodePtr][8] | byte[CodePtr][9] << 8   ' length of file
    ByteCount := NumberBytes                            ' number of bytes to send
    BitsOut(ByteCount >> 2, 32)
    repeat ByteCount                                    'send bytes    
      'BitsOut(byte[CodePtr++], 8)                      ' original code sent from hub memory
      BitsOut(fat.readbyte,8)                           ' read byte from SD card and send out
    if WaitBit(25)                                          'allow 250ms for positive checksum response    
      text.str(string("Error Checksum"))
  PrintStringCR(string("File sent"))
    'eeprom program command - commented out as not used 
    'if Command > 1
    '
    '  'allow 5s for positive program response
    '  if WaitBit(500)
    '    abort ErrorProgram
    '    
    '  'allow 2s for positive verify response
    '  if WaitBit(200)
    '    abort ErrorVerify

PRI IterateLFSR : Bit
  'get return bit
  Bit := LFSR & 1
  'iterate LFSR (8-bit, $B2 taps)
  LFSR := LFSR << 1 | (LFSR >> 7 ^ LFSR >> 5 ^ LFSR >> 4 ^ LFSR >> 1) & 1

PRI WaitBit(Hundredths) : Bit | PriorEcho
  repeat Hundredths
    'output 1t pulse                        
    BitsOut(1, 1)
    'sample bit and echo
    Bit := ina[P30]
    PriorEcho := Echo
    'output 2t pulse
    BitsOut(0, 1)
    'if echo was low, got bit                                      
    if not PriorEcho
      return
    'wait 10ms
    waitcnt(clkfreq / 100 + cnt)
  'timeout, abort
  text.str(string("Error Timeout"))

  
PRI BitsOut(Value, Bits)
  repeat Bits
    if Value & 1
      'output '1' (1t pulse)
      outa[P31] := 0                        
      Echo := ina[P30]
      outa[P31] := 1
    else
      'output '0' (2t pulse)
      outa[P31] := 0
      outa[P31] := 0
      Echo := ina[P30]
      Echo := ina[P30]
      outa[P31] := 1
    Value >>= 1
         